//==========================================================================
//
//      cml_serial.c
//
//
//==========================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//==========================================================================

#include <pkgconf/io_serial.h>
#include <pkgconf/io.h>

#include <cyg/io/io.h>
#include <cyg/hal/hal_intr.h>
#include <cyg/hal/plf_io.h>
#include <cyg/io/devtab.h>
#include <cyg/infra/diag.h>
#include <cyg/io/serial.h>

#ifdef CYGPKG_IO_SERIAL_CAMELOT
#define	UART_CLK	HAL_CAMELOT_SYSCLK	//CYGNUM_HAL_RTC_CLOCK
#define URCS_INIT	(URCS_TIE|URCS_RIE)

typedef struct cml_serial_info {
    CYG_ADDRWORD   base;
    CYG_WORD       int_num;
    cyg_interrupt  serial_interrupt;
    cyg_handle_t   serial_interrupt_handle;
} cml_serial_info;

static bool cml_serial_init(struct cyg_devtab_entry *tab);
static bool cml_serial_putc(serial_channel *chan, unsigned char c);
static Cyg_ErrNo cml_serial_lookup(struct cyg_devtab_entry **tab, 
                                   struct cyg_devtab_entry *sub_tab,
                                   const char *name);
static unsigned char cml_serial_getc(serial_channel *chan);
static Cyg_ErrNo cml_serial_set_config(serial_channel *chan, cyg_uint32 key,
                                         const void *xbuf, cyg_uint32 *len);
static void cml_serial_start_xmit(serial_channel *chan);
static void cml_serial_stop_xmit(serial_channel *chan);

static cyg_uint32 cml_serial_ISR(cyg_vector_t vector, cyg_addrword_t data);
static void       cml_serial_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data);

static SERIAL_FUNS(cml_serial_funs, 
                   cml_serial_putc, 
                   cml_serial_getc,
                   cml_serial_set_config,
                   cml_serial_start_xmit,
                   cml_serial_stop_xmit
    );


#if CYGPKG_IO_SERIAL_CAMELOT_SERIAL_0
static cml_serial_info cml_serial_info0 ={ UR_BASE, IRQ_UR};
static volatile unsigned int*up = (unsigned int*) UR_BASE ;

#if CYGNUM_IO_SERIAL_CAMELOT_SERIAL_0_BUFSIZE > 0
static unsigned char cml_serial_out_buf0[CYGNUM_IO_SERIAL_CAMELOT_SERIAL_0_BUFSIZE];
static unsigned char cml_serial_in_buf0[CYGNUM_IO_SERIAL_CAMELOT_SERIAL_0_BUFSIZE];

static SERIAL_CHANNEL_USING_INTERRUPTS(cml_serial_channel0,
                                       cml_serial_funs, 
                                       cml_serial_info0,
                                       CYG_SERIAL_BAUD_RATE(CYGNUM_IO_SERIAL_CAMELOT_SERIAL_0_BAUD),
                                       CYG_SERIAL_STOP_DEFAULT,
                                       CYG_SERIAL_PARITY_DEFAULT,
                                       CYG_SERIAL_WORD_LENGTH_DEFAULT,
                                       CYG_SERIAL_FLAGS_DEFAULT,
                                       &cml_serial_out_buf0[0], 
                                       sizeof(cml_serial_out_buf0),
                                       &cml_serial_in_buf0[0], 
                                       sizeof(cml_serial_in_buf0)
    );
#else
static SERIAL_CHANNEL(cml_serial_channel0,
                      cml_serial_funs, 
                      cml_serial_info0,
                      CYG_SERIAL_BAUD_RATE(CYGNUM_IO_SERIAL_CAMELOT_SERIAL_0_BAUD),
                      CYG_SERIAL_STOP_DEFAULT,
                      CYG_SERIAL_PARITY_DEFAULT,
                      CYG_SERIAL_WORD_LENGTH_DEFAULT,
                      CYG_SERIAL_FLAGS_DEFAULT
    );
#endif

DEVTAB_ENTRY(cml_serial_io0, 
             CYGDAT_IO_SERIAL_CAMELOT_SERIAL_0_NAME,
             0,                 // Does not depend on a lower level interface
             &cyg_io_serial_devio, 
             cml_serial_init, 
             cml_serial_lookup,     // Serial driver may need initializing
             &cml_serial_channel0
    );
    
#endif // !!CYGPKG_IO_SERIAL_CAMELOT_SERIAL0


// Internal function to actually configure the hardware to desired baud rate, etc.
static bool
cml_serial_config_port(serial_channel *chan, cyg_serial_info_t *new_config, bool init)
{
    cml_serial_info *cml_chan = (cml_serial_info *)chan->dev_priv;
    cyg_uint16 baud_divisor;
    cyg_int32 parity, wordlen, stopbit;
    cyg_uint32 int_mask;
#if 0
    if(!new_config->baud)
	return false;
    baud_divisor = UART_CLK/new_config->baud;
#else
    baud_divisor = UART_CLK/115200;
#endif
    if (baud_divisor == 0)
        return false;

    up[URCS>>2] = baud_divisor<<16 | URCS_INIT;
    
    if (new_config != &chan->config) {
        chan->config = *new_config;
    }
    return true;
}

// Function to initialize the device.  Called at bootstrap time.
static bool 
cml_serial_init(struct cyg_devtab_entry *tab)
{
    serial_channel *chan = (serial_channel *)tab->priv;
    cml_serial_info *cml_chan = (cml_serial_info *)chan->dev_priv;
#ifdef CYGDBG_IO_INIT
    diag_printf("cml SERIAL init - dev: %x.%d\n", cml_chan->base, cml_chan->int_num);
#endif
    (chan->callbacks->serial_init)(chan);  // Really only required for interrupt driven devices
    if (chan->out_cbuf.len != 0) {
        cyg_drv_interrupt_create(cml_chan->int_num,
                                 0,         // can change IRQ0 priority
                                 (cyg_addrword_t)chan,   //  Data item passed to interrupt handler
                                 cml_serial_ISR,
                                 cml_serial_DSR,
                                 &cml_chan->serial_interrupt_handle,
                                 &cml_chan->serial_interrupt);
        cyg_drv_interrupt_attach(cml_chan->serial_interrupt_handle);
        cyg_drv_interrupt_unmask(cml_chan->int_num);
    }
    cml_serial_config_port(chan, &chan->config, true);
    return true;
}

// This routine is called when the device is "looked" up (i.e. attached)
static Cyg_ErrNo 
cml_serial_lookup(struct cyg_devtab_entry **tab, 
                  struct cyg_devtab_entry *sub_tab,
                  const char *name)
{
    serial_channel *chan = (serial_channel *)(*tab)->priv;
    (chan->callbacks->serial_init)(chan);  // Really only required for interrupt driven devices
    return ENOERR;
}

// Send a character to the device output buffer.
// Return 'true' if character is sent to device
static bool
cml_serial_putc(serial_channel *chan, unsigned char c)
{
    if (!(up[URCS>>2] & URCS_TF))
    {
    	up[URBR] = c<<24;
	return true;
    }
    return false;
    
}

// Fetch a character from the device input buffer, waiting if necessary
static unsigned char 
cml_serial_getc(serial_channel *chan)
{
    do {
	if (up[URCS>>2] & URCS_RF)
	    return up[URBR]>>24;
    } while (1);
    return 0;
}

// Set up the device characteristics; baud rate, etc.
static Cyg_ErrNo
cml_serial_set_config(serial_channel *chan, cyg_uint32 key,
                        const void *xbuf, cyg_uint32 *len)
{
    switch (key) {
    case CYG_IO_SET_CONFIG_SERIAL_INFO:
      {
        cyg_serial_info_t *config = (cyg_serial_info_t *)xbuf;
        if ( *len < sizeof(cyg_serial_info_t) ) {
            return -EINVAL;
        }
        *len = sizeof(cyg_serial_info_t);
        if ( true != cml_serial_config_port(chan, config, false) )
            return -EINVAL;
      }
      break;
    default:
        return -EINVAL;
    }
    return ENOERR;
}

// Enable the transmitter on the device
static void
cml_serial_start_xmit(serial_channel *chan)
{
    up[URCS>>2] |= URCS_TIE; 
    (chan->callbacks->xmt_char)(chan);
}

// Disable the transmitter on the device
static void 
cml_serial_stop_xmit(serial_channel *chan)
{
    up[URCS>>2] &= ~URCS_TIE; 
}

// Serial I/O - low level interrupt handler (ISR)
static cyg_uint32 
cml_serial_ISR(cyg_vector_t vector, cyg_addrword_t data)
{
    serial_channel *chan = (serial_channel *)data;
    cml_serial_info *cml_chan = (cml_serial_info *)chan->dev_priv;

    cyg_drv_interrupt_mask(cml_chan->int_num);
    cyg_drv_interrupt_acknowledge(cml_chan->int_num);
    return CYG_ISR_CALL_DSR;  // Cause DSR to be run
}

// Serial I/O - high level interrupt handler (DSR)
static void       
cml_serial_DSR(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
    serial_channel *chan = (serial_channel *)data;
    cml_serial_info *cml_chan = (cml_serial_info *)chan->dev_priv;
    cyg_uint32 int_status, c;

    int_status = up[URCS>>2];
    
    if(!(int_status & URCS_TF)) 
        (chan->callbacks->xmt_char)(chan);
        
    if(int_status & URCS_RF) 
    {
    	do{
    	    c = up[URBR];
    	    (chan->callbacks->rcv_char)(chan, c>>24);
        } while( up[URCS>>2]& URCS_RF);
    }
    cyg_drv_interrupt_unmask(cml_chan->int_num);
}

#endif

//-------------------------------------------------------------------------
// EOF cml_serial.c
