#ifndef CYGONCE_HAL_PLF_INTR_H
#define CYGONCE_HAL_PLF_INTR_H

//==========================================================================
//
//      plf_intr.h
//
//      JMR3904 Interrupt and clock support
//
//==========================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    nickg
// Contributors: nickg, jskov,
//               gthomas, jlarmour
// Date:         1999-02-16
// Purpose:      Define Interrupt support
// Description:  The macros defined here provide the HAL APIs for handling
//               interrupts and the clock for the JMR3904 board.
//              
// Usage:
//              #include <cyg/hal/plf_intr.h>
//              ...
//              
//
//####DESCRIPTIONEND####
//
//==========================================================================

#include <pkgconf/hal.h>

#include <cyg/infra/cyg_type.h>

//--------------------------------------------------------------------------
// Control-C support.

#if defined(CYGDBG_HAL_MIPS_DEBUG_GDB_CTRLC_SUPPORT)

# define CYGHWR_HAL_GDB_PORT_VECTOR CYGNUM_HAL_INT_UART

externC cyg_uint32 hal_ctrlc_isr(CYG_ADDRWORD vector, CYG_ADDRWORD data);

# define HAL_CTRLC_ISR hal_ctrlc_isr

#endif
//
//--------------------------------------------------------------------------
// Interrupt vectors.

#ifndef CYGHWR_HAL_INTERRUPT_VECTORS_DEFINED

// These are decoded via the IP bits of the cause
// register when an external interrupt is delivered.

#define CYGNUM_HAL_INT_0                0
#define CYGNUM_HAL_INT_1                1
#define CYGNUM_HAL_INT_2                2
#define CYGNUM_HAL_INT_MAC              3
#define CYGNUM_HAL_INT_UART	        4   
#define CYGNUM_HAL_INT_TMR_2         	5   
#define CYGNUM_HAL_INT_TMR_1            6
#define CYGNUM_HAL_INT_TMR_0            7

// Min/Max ISR numbers and how many there are
#define CYGNUM_HAL_ISR_MIN                     0
#define CYGNUM_HAL_ISR_MAX                     15
#define CYGNUM_HAL_ISR_COUNT                   16

// The vector used by the Real time clock
#define CYGNUM_HAL_INTERRUPT_RTC            CYGNUM_HAL_INT_TMR_1

#define CYGHWR_HAL_INTERRUPT_VECTORS_DEFINED

#endif


//--------------------------------------------------------------------------
// Interrupt controller access.

#ifndef CYGHWR_HAL_INTERRUPT_CONTROLLER_ACCESS_DEFINED

#if defined(CYGPKG_HAL_MIPS_CAMELOT)

// Array which stores the configured priority levels for the configured
// interrupts.
externC volatile CYG_BYTE hal_interrupt_level[CYGNUM_HAL_ISR_COUNT];


#define HAL_INTERRUPT_MASK( intr )                       \
{	\
	register volatile int *ic=(int*) IC_BASE ; \
  	ic[IMSK] |= (1 << (intr)); \
}

#define HAL_INTERRUPT_UNMASK( intr )                                \
{	\
	register volatile int *ic=(int*) IC_BASE ; \
  	ic[IMSK] &= ~(1 << (intr)); \
}

#define HAL_INTERRUPT_ACKNOWLEDGE( intr )                           \
{	\
	register volatile int *ic=(int*) IC_BASE ; \
  	ic[ISTS] = (1 << (intr)); \
	(void)ic[ISTS]; \
}

#define HAL_INTERRUPT_CONFIGURE( _vector_, _level_, _up_ )              \
{                                                                       \
}

#define HAL_INTERRUPT_SET_LEVEL( _vector_, _level_ )    \
{                                                       \
}

	
#define CYGHWR_HAL_INTERRUPT_CONTROLLER_ACCESS_DEFINED

#else

#error Unspecified CAMELOT variant

#endif

#endif //CYGHWR_HAL_INTERRUPT_CONTROLLER_ACCESS_DEFINED

//--------------------------------------------------------------------------
// Clock control

#ifndef CYGHWR_HAL_CLOCK_CONTROL_DEFINED

externC CYG_WORD32 cyg_hal_clock_period;
#define CYGHWR_HAL_CLOCK_PERIOD_DEFINED

#if defined(CYGPKG_HAL_MIPS_CAMELOT)

#define	ISTS_VAL (1<<IRQ_TMR1)			/* timer1 */
#define	TCN_INI	 (1<<2)|(1<<1)|(1<<0)	/* b2: go, b1: ie, b0: auto-reload */

#define HAL_CLOCK_INITIALIZE( _period_ )                             \
{                                                                    \
  volatile unsigned int *pt = (unsigned int *) TM_BASE;	\
  cyg_hal_clock_period = HAL_CAMELOT_TMR_PERIOD ;		\
  pt[T1LR] = cyg_hal_clock_period; 			\
  pt[T1PR] = CYGNUM_HAL_RTC_PRESCALE -1 ;		\
  pt[T1CN] = TCN_INI;					\
}

#define HAL_CLOCK_RESET( _vector_, _period_ )                        \
{                                                       \
  volatile unsigned int *pt = (unsigned int *) TM_BASE;	\
  pt[T1IS]=1;						\
}

# define HAL_CLOCK_READ( _pvalue_ )                                     \
{							\
  volatile unsigned int *pt = (unsigned int *) TM_BASE;	\
  *(_pvalue_) = pt[T1CR] ;				\
}


#define CYGHWR_HAL_CLOCK_CONTROL_DEFINED

#else //CYGPKG_HAL_MIPS_CAMELOT

#error Unspecified CAMELOT variant

#endif

#endif //CYGHWR_HAL_CLOCK_CONTROL_DEFINED

#if defined(CYGVAR_KERNEL_COUNTERS_CLOCK_LATENCY) && \
    !defined(HAL_CLOCK_LATENCY)
#define HAL_CLOCK_LATENCY( _pvalue_ )                   \
CYG_MACRO_START                                         \
    register CYG_WORD32 _cval_;                         \
    HAL_CLOCK_READ(&_cval_);                            \
    *(_pvalue_) = _cval_ - cyg_hal_clock_period;        \
CYG_MACRO_END
#endif

//----------------------------------------------------------------------------
// Reset.
#ifndef CYGHWR_HAL_RESET_DEFINED
extern void hal_camelot_reset( void );
#define CYGHWR_HAL_RESET_DEFINED

#define HAL_PLATFORM_RESET()     hal_camelot_reset()     
#define HAL_PLATFORM_RESET_ENTRY 0xbfc00000

#endif // CYGHWR_HAL_RESET_DEFINED

//--------------------------------------------------------------------------
#endif // ifndef CYGONCE_HAL_PLF_INTR_H

// End of plf_intr.h
